require 'startmeup/version'
require 'fileutils'
require 'pathname'
require 'thor'

module StartMeUp
  class Generator < Thor
    map ['-v', '--version'] => :version
    map ['scaffold'] => :install
    map ['delete'] => :remove

    desc 'install', 'Install StartMeUp into your project'
    method_options path: :string
    def install
      if startmeup_files_already_exist?
        puts "StartMeUp files already installed, doing nothing."
      else
        install_files
        print <<-postinstall
*************************************************

STYLES
StartMeUp files installed to #{install_path}/base

DEBUG
_debug.haml partial added to views/shared
(add it after the body tag in your application layout)

*************************************************
        postinstall
      end
    end

    desc 'reset', 'Reset StartMeUp'
    def reset
      if startmeup_files_already_exist?
        remove_startmeup_directory
        install_files
        puts "StartMeUp files updated."
      else
        puts "No existing StartMeUp installation. Doing nothing."
      end
    end

    desc 'remove', 'Remove StartMeUp'
    def remove
      if startmeup_files_already_exist?
        remove_startmeup_directory
        puts "StartMeUp was successfully removed."
      else
        puts "No existing StartMeUp installation. Doing nothing."
      end
    end

    desc 'version', 'Show StartMeUp version'
    def version
      say "StartMeUp #{StartMeUp::VERSION}"
    end

    private

    def startmeup_files_already_exist?
      File.directory?(install_path)
    end

    def install_path
      Pathname.new(options[:path].to_s).join('base')
    end

    def views_install_path
      pn = Pathname.new(options[:path].to_s).join('base')
      pn.parent.parent.parent + 'views'
    end

    def install_files
      FileUtils.mkdir_p(install_path)
      FileUtils.mkdir_p(views_install_path)
      FileUtils.cp_r(all_stylesheets, install_path)
      FileUtils.cp_r(all_views, views_install_path)
    end

    def remove_startmeup_directory
      FileUtils.rm_rf("base")
    end

    def all_stylesheets
      Dir["#{stylesheets_directory}/*"]
    end

    def all_views
      Dir["#{views_directory}/*"]
    end

    def stylesheets_directory
      File.join(top_level_directory, "app", "assets", "stylesheets")
    end

    def views_directory
      File.join(top_level_directory, "app", "views")
    end

    def top_level_directory
      File.dirname(File.dirname(File.dirname(__FILE__)))
    end
  end
end
